/*********************************************************************NVMH3****
File:  $Id: //sw/devtools/SDK/9.5/SDK/MEDIA/HLSL/displace2.fx#1 $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

******************************************************************************/

float Script : STANDARDSGLOBAL <
    string UIWidget = "none";
    string ScriptClass = "object";
    string ScriptOrder = "standard";
    string ScriptOutput = "color";
    string Script = "Technique=Displace;";
> = 0.8;

#define TIMER

float4x4 WorldITXf : WorldInverseTranspose <string UIWidget="None";>;
float4x4 WvpXf : WorldViewProjection <string UIWidget="None";>;
float4x4 WorldXf : World <string UIWidget="None";>;
float4x4 ViewIXf : ViewInverse <string UIWidget="None";>;
#ifdef TIMER
float Timer : TIME <string UIWidget="None";>;
//float Timer <
//    string UIWidget = "slider";
//    float UIMin = 0.0f;
//    float UIMax = 10.0f;
//    float UIStep = 0.01f;
//> = 0.4f;
#endif

///////////////////////////////////////////////////////////
/////////////////////////////////////// Tweakables ////////
///////////////////////////////////////////////////////////

float3 LightDirD : Direction
<
    string UIName = "Light Direction"; 
    string Object = "DirectionalLight";
    string Space = "World";
> = {-10.0f, 15.0f, 30.0f};

#ifdef TIMER
float SpeedS <
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 1.0f;
    float UIStep = 0.01f;
> = 0.4f;

float TexScale <
	string UIName = "Texture Scale";
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 10.0;
	float UIStep = 0.01;
> = 1.0f;

/*float SpeedT <
    string UIWidget = "slider";
    float UIMin = -1.0f;
    float UIMax = 4.0f;
    float UIStep = 0.01f;
> = 2.f;*/
#endif

//////////////////////

texture DiffTex : Diffuse <
	string ResourceName = "default_color.dds";
	string TextureType = "2D";
>;

sampler2D DiffSamp = sampler_state
{
	Texture = <DiffTex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

float Wrap <
	string UIName = "Wrap";
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 1.0;
	float UIStep = 0.01;
> = 0.5f;

//////////////////////

float Bumpy <
	string UIName = "NormalMap";
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 4.0;
	float UIStep = 0.01;
> = 0.5f;

float DHeight <
	string UIName = "Displacement";
	string UIWidget = "slider";
	float UIMin = -2.0;
	float UIMax = 2.0;
	float UIStep = 0.01;
> = 0.5f;

float Ks
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.5;
    float UIStep = 0.01;
    string UIName = "Specular";
> = 1.0;


float SpecExpon : SpecularPower
<
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName = "Specular power";
> = 12.0;

//////////////////////

texture BumpTex <
	string ResourceName = "default_bump_normal.dds";
	string TextureType = "2D";
>;

sampler BumpSamp = sampler_state
{
	Texture = <BumpTex>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

texture DispTex <
	string ResourceName = "default_bump_R32F.dds";
	string TextureType = "2D";
	//string Format = "r32f";
	//string Format = "a32b32g32r32f";
>;

sampler DispSamp = sampler_state
{
	Texture = <DispTex>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

///////////////////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

// used for all other passes
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		: TEXCOORD0;
    float3 N		: TEXCOORD1;
    float3 T		: TEXCOORD2;
    float3 B		: TEXCOORD3;
    float3 V		: TEXCOORD4;
	float4 DiffCol : COLOR0;
};

//////////////////////////////////////////////////////
////////////////////////////////// pixel shaders /////
//////////////////////////////////////////////////////

vertexOutput pushVS(appdata IN,uniform float3 LightDir)
{
    vertexOutput OUT;
	float2 nuv = IN.UV.xy;
#ifdef TIMER
	float a = Timer * SpeedS;
	float c = cos(a); float s = sin(a);
	nuv = nuv - float2(0.5,0.5);
	nuv = float2(nuv.x*c-nuv.y*s,
				 nuv.y*c+nuv.x*s);
	nuv = nuv + float2(0.5,0.5);
#endif
    nuv= frac(nuv);
    OUT.UV = nuv;
	float4 tx = tex2Dlod(DispSamp,float4(nuv,0,0));
	float tv = TexScale * tx.x;
	float d = DHeight * (tv-0.5);
    float3 Nn = normalize(mul(IN.Normal, WorldITXf).xyz);
	OUT.N = Nn;
    OUT.T = normalize(mul(IN.Tangent, WorldITXf).xyz);
    OUT.B = normalize(mul(IN.Tangent, WorldITXf).xyz);
    float4 No = normalize(IN.Normal);
    float4 dNo = d * No;
    float4 Po = float4(IN.Position.xyz,1) + dNo;	// obj coords
    float4 Pw = mul(Po,WorldXf);			// world coordinates
    OUT.V = normalize(ViewIXf[3].xyz - Pw.xyz);	// obj coords
	float ldn = dot(normalize(LightDir),Nn);
    float4 Ph = mul(Po, WvpXf);
    OUT.HPosition = Ph;
    float2 fadeuv = lerp(nuv,tv.xx,0.5);
     fadeuv = lerp(fadeuv,(float2(1,1)),0.5);
	//OUT.DiffCol = float4(2.0*abs(dNo.xyz-0.5),1); // float4(max(0,ldn).xxx,1.0);
	//OUT.DiffCol = float4((0.5*(ldn+1)).xxx,1); // float4(max(0,ldn).xxx,1.0);
	//OUT.DiffCol = float4(tv.xxx,1); // float4(max(0,ldn).xxx,1.0);
	OUT.DiffCol = float4(nuv.xy,tv,1); // float4(max(0,ldn).xxx,1.0);
	//OUT.DiffCol = float4((0.5*(ldn+1)).xxx*float3(fadeuv,0),1); // float4(max(0,ldn).xxx,1.0);
	// OUT.DiffCol = float4(nuv,0,1); // float4(max(0,ldn).xxx,1.0);
    return OUT;
}

float4 nPS(vertexOutput IN) : COLOR
{
	float3 Nn = normalize(IN.N);
	float3 Tn = normalize(IN.T);
	float3 Bn = normalize(IN.B);
	float2 bumps = Bumpy * (tex2D(DiffSamp,IN.UV).xy - float2(0.5,0.5));
	Nn += (Tn * bumps.x + Bn * bumps.y);
	Nn = normalize(Nn);
	float3 Ln = normalize(-LightDirD);
	float ldn = dot(Ln,Nn);
	float grad = (ldn + Wrap) / (1.0 + Wrap);
    float3 Vn = normalize(IN.V);
    float3 Hn = normalize(Vn + Ln);
    float hdn = dot(Hn,Nn);
    float4 litVec = lit(grad,hdn,SpecExpon);
	float4 tc = tex2D(DiffSamp,IN.UV);
	float4 diff = litVec.y*tc;
	float spec = Ks * litVec.z;
	return float4(diff + float4(spec.xxx,0.0));
}

////////////////////////////////////////////////////////////
/////////////////////////////////////// techniques /////////
////////////////////////////////////////////////////////////

technique Displace <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
> {
		VertexShader = compile vs_3_0 pushVS(-LightDirD);
		cullmode = none;
		ZEnable = true;
		PixelShader  = compile ps_3_0 nPS();
    }
}
